def ping():
    return 'pong'


def load_dependency(package_name):                                              # Load a single dependency from S3
    import boto3, zipfile, sys, io, os

    sts         = boto3.client('sts')                                           # Get account and region for bucket name
    account_id  = sts.get_caller_identity()['Account']
    region_name = boto3.session.Session().region_name
    bucket_name = f'{account_id}--osbot-lambdas--{region_name}'
    s3_key      = f'lambdas-dependencies/{package_name}.zip'
    temp_folder = f'/tmp/lambdas-dependencies/{package_name}'

    s3          = boto3.client('s3')
    response    = s3.get_object(Bucket=bucket_name, Key=s3_key)                 # Download dependency
    zip_bytes   = response['Body'].read()

    os.makedirs(temp_folder, exist_ok=True)
    with zipfile.ZipFile(io.BytesIO(zip_bytes), 'r') as zip_ref:                # extract it to dependencies folder
        zip_ref.extractall(temp_folder)

    sys.path.append(temp_folder)
    return f'{package_name} (loaded from S3)'


def load_dependencies(packages):                                        # Load multiple dependencies - just calls load_dependency for each
    if isinstance(packages, str):                                       # Ensure packages is a list
        packages = [packages]

    return [load_dependency(package_name) for package_name in packages]


from osbot_aws.aws.lambda_.dependencies.Lambda__Dependency__Local       import Lambda__Dependency__Local
from osbot_aws.aws.lambda_.dependencies.Lambda__Dependency__S3          import Lambda__Dependency__S3
from osbot_aws.aws.lambda_.schemas.Safe_Str__File__Name__Python_Package import Safe_Str__File__Name__Python_Package
from osbot_utils.type_safe.Type_Safe                                    import Type_Safe


class Lambda__Dependency(Type_Safe):
    package_name      : Safe_Str__File__Name__Python_Package  = None
    dependency__local : Lambda__Dependency__Local             = None
    dependency__s3    : Lambda__Dependency__S3                = None

    def __init__(self, package_name):
        super().__init__(package_name=package_name)
        self.dependency__local = Lambda__Dependency__Local(package_name=package_name)
        self.dependency__s3    = Lambda__Dependency__S3   (package_name=package_name)


from unittest                                                 import TestCase
from osbot_aws.aws.s3.S3__DB_Base                             import S3__DB_Base
from osbot_aws.testing.Temp__Random__AWS_Credentials          import Temp__Random__AWS_Credentials
from tests.integration.osbot_aws__objs_for__integration_tests import setup__osbot_aws__integration_tests


class TestCase__S3__Temp_DB(TestCase):

    @classmethod
    def setUpClass(cls):
        setup__osbot_aws__integration_tests()
        #cls.local_stack = Local_Stack().activate()
        cls.random_aws_creds  = Temp__Random__AWS_Credentials().with_default_credentials()
        cls.s3_db_base        = S3__DB_Base()
        cls.s3_db             = cls.s3_db_base                  # todo: see if this is a better name to use when using this class
        with cls.s3_db_base as _:
            assert _.using_local_stack() is True
            _.setup()                                           # this will create the temp bucket
            assert _.bucket_exists() is True

    @classmethod
    def tearDownClass(cls):
        with cls.s3_db_base as _:
            #assert _.using_local_stack      () is True
            assert _.bucket_delete_all_files() is True
            assert _.bucket_delete          () is True
        cls.random_aws_creds.restore_vars()

        #cls.local_stack.deactivate()


import random
import string
from osbot_utils.utils.Env              import get_env
from osbot_utils.testing.Temp_Env_Vars  import Temp_Env_Vars

OSBOT_AWS__LOCAL_STACK__AWS_ACCOUNT_ID     = '000000000000'               # default local-stack account id for lambdas
OSBOT_AWS__LOCAL_STACK__AWS_DEFAULT_REGION = 'us-east-1'                  # default local-stack region for lambdas


class Temp__Random__AWS_Credentials(Temp_Env_Vars):
    AWS_ACCESS_KEY_ID     : str = None
    AWS_SECRET_ACCESS_KEY : str = None
    AWS_ACCOUNT_ID        : str = None
    AWS_DEFAULT_REGION    : str = None

    def __enter__(self):
        self.with_default_credentials()         # this will call the super().set_vars()
        return self


    def with_localstack_credentials(self):
        self.AWS_ACCOUNT_ID        = OSBOT_AWS__LOCAL_STACK__AWS_ACCOUNT_ID
        self.AWS_DEFAULT_REGION    = OSBOT_AWS__LOCAL_STACK__AWS_DEFAULT_REGION
        self.AWS_ACCESS_KEY_ID     = self.random_aws_access_key_id()
        self.AWS_SECRET_ACCESS_KEY = self.random_aws_secret_access_key()
        self.set_aws_env_vars()
        return self

    def with_default_credentials(self):         # Set default credentials, using existing values or generating new ones.
        if not self.AWS_ACCESS_KEY_ID    : self.AWS_ACCESS_KEY_ID     = get_env('AWS_ACCESS_KEY_ID'    ) or self.random_aws_access_key_id    ()       # Generate new values only if not already set
        if not self.AWS_SECRET_ACCESS_KEY: self.AWS_SECRET_ACCESS_KEY = get_env('AWS_SECRET_ACCESS_KEY') or self.random_aws_secret_access_key()
        if not self.AWS_ACCOUNT_ID       : self.AWS_ACCOUNT_ID        = get_env('AWS_ACCOUNT_ID'       ) or self.random_aws_account_id       ()
        if not self.AWS_DEFAULT_REGION   : self.AWS_DEFAULT_REGION    = get_env('AWS_DEFAULT_REGION'   ) or self.random_aws_region           ()

        self.set_aws_env_vars()
        return self

    def set_aws_env_vars(self):                         # Update env_vars with the credentials
        aws_credentials = { 'AWS_ACCESS_KEY_ID'     : self.AWS_ACCESS_KEY_ID    ,
                            'AWS_SECRET_ACCESS_KEY' : self.AWS_SECRET_ACCESS_KEY,
                            'AWS_ACCOUNT_ID'        : self.AWS_ACCOUNT_ID       ,
                            'AWS_DEFAULT_REGION'    : self.AWS_DEFAULT_REGION   }
        self.env_vars.update(aws_credentials)
        self.set_vars()
        return self

    def random_aws_access_key_id(self):
        return 'AKIA' + ''.join(random.choices(string.ascii_uppercase + string.digits, k=16))       # AWS access key IDs are typically 20 characters long, uppercase letters and digits

    def random_aws_secret_access_key(self):
        return ''.join(random.choices(string.ascii_letters + string.digits + '/+=', k=40))          # AWS secret access keys are typically 40 characters long, mix of letters, digits, and special characters

    def random_aws_account_id(self):
        return ''.join(random.choices(string.digits, k=12))                                         # AWS account IDs are typically 12 digits long

    def random_aws_region(self):
        regions = [ 'us-west-1'     , 'us-west-2'   ,                       # 'us-east-1' # note: not using us-east-1 since it was causing some side effects with Local_Stack bucket creation (InvalidLocationConstraint error)
                    'eu-west-1'     , 'eu-central-1', 'ap-southeast-1',
                    'ap-northeast-1', 'ap-south-1'  , 'sa-east-1'     ]
        return random.choice(regions)                                                               # Randomly select from a list of common AWS regions


Temp_AWS_Credentials = Temp__Random__AWS_Credentials


from osbot_utils.type_safe.Type_Safe import Type_Safe


class Session__Kwargs(Type_Safe):
    aws_access_key_id       : str = None                                # todo: refactor these client setup details into a common base class since (this is should be usable by all osbot-aws boto3 wrapper classes)
    aws_secret_access_key   : str = None
    endpoint_url            : str = None
    region_name             : str = None


from typing                                                             import List, Optional, Dict, Any
from osbot_aws.aws.s3.S3__DB_Base                                       import S3__DB_Base
from osbot_utils.decorators.methods.cache_on_self                       import cache_on_self
from osbot_utils.helpers.llms.cache.Virtual_Storage__Local__Folder      import Virtual_Storage__Local__Folder
from osbot_utils.helpers.safe_str.Safe_Str__File__Path                  import Safe_Str__File__Path
from osbot_utils.utils.Files                                            import path_combine_safe

VIRTUAL_STORAGE__DEFAULT__ROOT_FOLDER = Safe_Str__File__Path("s3-virtual-storage/")

class Virtual_Storage__S3(Virtual_Storage__Local__Folder):                                          # Storage implementation using AWS S3 as the backend."""
    root_folder: Safe_Str__File__Path = VIRTUAL_STORAGE__DEFAULT__ROOT_FOLDER                     # Prefix for all stored files in S3
    s3_db      :  S3__DB_Base

    def bucket_name(self):
        return self.s3_db.s3_bucket()

    def folder__create(self, path_folder) -> None:                                                  # Folders don't need to be explicitly created in S3.
        pass                                                                                        # S3 has no concept of folders, they're created implicitly with object keys

    def json__load(self, path: Safe_Str__File__Path) -> Optional[Dict[str, Any]]:                   # Load JSON data from S3
        s3_key = self.get_s3_key(path)
        if self.file__exists(path):
            json_data = self.s3_db.s3_file_contents_json(s3_key)
            return json_data
        return None

    def json__save(self, path: Safe_Str__File__Path, data: dict) -> bool:           # Save JSON data to S3
        s3_key      = self.get_s3_key(path)
        result      = self.s3_db.s3_save_data(data=data, s3_key=s3_key)
        return result is not None

    def get_full_path(self, path: Safe_Str__File__Path) -> Safe_Str__File__Path:    # For S3, we don't need physical paths, but we maintain the same interface.
        return path

    def get_s3_key(self, path: Safe_Str__File__Path) -> str:                        # Convert the virtual path to an S3 key with proper prefix.
        if str(path).startswith(str(self.root_folder)):
            return str(path)
        return path_combine_safe(self.root_folder, path)

    def file__delete(self, path: Safe_Str__File__Path) -> bool:             # Delete a file from S3.
        s3_key = self.get_s3_key(path)
        return self.s3_db.s3_file_delete(s3_key)

    def file__exists(self, path: Safe_Str__File__Path) -> bool:             # Check if file exists in S3.
        s3_key = self.get_s3_key(path)
        return self.s3_db.s3_file_exists(s3_key)

    def files__all(self, full_path=False) -> List[str]:                                      # List all files in S3 with the specified prefix.
        prefix = str(self.root_folder)
        if prefix.endswith('/'):                                                             # todo: handle bug in s3_folder_files__all which will lost the first char if the folder path ends with an /
            prefix = prefix[:-1]
        all_objects = self.s3_db.s3_folder_files__all(folder=prefix, full_path=full_path)
        return all_objects

    @cache_on_self
    def path_folder__root_cache(self) -> str:                               # In S3 storage, this is a virtual concept.
        return str(self.root_folder)

    def clear_all(self) -> bool:                                            # Clear all stored files in this virtual storage.
        raise NotImplemented("not implemented since this is quite a dangerous method")

    # # todo: check for the performance impact (and costs) of this method
    # def stats(self) -> Dict[str, Any]:                                      # Get storage statistics.
    #
    #     files = self.files__all(full_path=True)
    #     total_size = 0
    #
    #     for file_path in files:
    #         file_info = self.s3_db.s3_file_info(file_path)
    #         if file_info and 'ContentLength' in file_info:
    #             total_size += file_info['ContentLength']
    #
    #     return { "storage_type"    : "s3"                   ,
    #              "bucket_name"     : self.s3_db.s3_bucket() ,
    #              "prefix"          : self.root_folder       ,
    #              "file_count"      : len(files)             ,
    #              "total_size_bytes": total_size             }


import boto3
from osbot_aws.AWS_Config                                     import AWS_Config
from osbot_aws.apis.Session                                   import URL__LOCAL_STACK__ENDPOINT_URL
from osbot_aws.aws.dynamo_db.Dynamo_DB                        import Dynamo_DB
from unittest                                                 import TestCase
from tests.integration.osbot_aws__objs_for__integration_tests import setup__osbot_aws__integration_tests

# TEST__AWS_ACCOUNT_ID        = '000011110000'
# TEST__AWS_DEFAULT_REGION    = 'eu-west-1'
# TEST__AWS_ACCESS_KEY_ID     = 'aaaaaaaaa'
# TEST__AWS_SECRET_ACCESS_KEY = 'bbbbbbbbb'
#URL_DOCKER__DYNAMODB__LOCAL = 'http://localhost:8000'

class TestCase__Dynamo_DB__Local(TestCase):

    @classmethod
    def setUpClass(cls):
        setup__osbot_aws__integration_tests()
        cls.aws_config = AWS_Config()
        # cls.aws_account_id          = get_env('AWS_ACCOUNT_ID'       , TEST__AWS_ACCOUNT_ID       )         # use existing values if they already exist
        # cls.aws_default_region      = get_env('AWS_DEFAULT_REGION'   , TEST__AWS_DEFAULT_REGION   )
        # cls.aws_access_key_id       = get_env('AWS_ACCESS_KEY_ID'    , TEST__AWS_ACCESS_KEY_ID    )         # this is neeed by create table
        # cls.aws_secret_access_key   = get_env('AWS_SECRET_ACCESS_KEY', TEST__AWS_SECRET_ACCESS_KEY)
        # tmp_vars_values             = dict(AWS_ACCOUNT_ID        = cls.aws_account_id       ,           # todo: find better way to do this, since this will impact all code executed in the current test
        #                                    AWS_DEFAULT_REGION    = cls.aws_default_region   ,
        #                                    AWS_ACCESS_KEY_ID     = cls.aws_account_id       ,
        #                                    AWS_SECRET_ACCESS_KEY = cls.aws_secret_access_key)
        # cls.tmp_env_vars            = Temp_Env_Vars(env_vars=tmp_vars_values).set_vars()
        # cls.hook_method             = Hook_Method(Dynamo_DB, 'client'     )              # todo: see if we also need to hook the main S3 class (from osbot_aws)
        # cls.hook_method.mock_call   = cls.dynamo_db__client
        # cls.hook_method.wrap()
        cls.dynamo_db               = Dynamo_DB()
        assert cls.dynamo_db.client().meta.endpoint_url == URL__LOCAL_STACK__ENDPOINT_URL

    # @classmethod
    # def tearDownClass(cls):
    #     cls.hook_method.unwrap()
    #     cls.tmp_env_vars.restore_vars()

    # def dynamo_db__client(self):
    #     return boto3.client('dynamodb', endpoint_url=URL_DOCKER__DYNAMODB__LOCAL)


# todo: see if this is still needed
# from osbot_utils.utils.Process import Process
#
#
# class Lambda_Docker:
#
#     def __init__(self):
#         self.docker_image = "lambci/lambda:python3.8"
#
#
#     def run_bash(self, cmd):
#         run_params = ["run", "--rm", "--entrypoint", "bash", self.docker_image, "-c",cmd]
#         return Process.run("docker", run_params)
#
#     # todo: finish this method
#     # def invoke(self, cmd):
#     #     run_params = ["run", "--rm",
#     #                   "--entrypoint", "bash",
#     #                   self.docker_image, "-c", cmd]
#     #     # docker run --rm                             \
#
#         #           -v "$PWD":/var/task:ro,delegated \
#         #           lambci/lambda:python3.8          \
#         #           lambda_function.run
#
